package hu.bme.mit.gamma.impl.crossroad;

import org.yakindu.scr.ITimer;

import hu.bme.mit.gamma.impl.interfaces.*;    	
import hu.bme.mit.gamma.impl.controller.*;
import hu.bme.mit.gamma.impl.trafficlightctrl.*;

public class CrossroadComponent implements CrossroadComponentInterface {			
	// ComponentDeclaration instances
	private TrafficLightCtrlDeclaration priority = new TrafficLightCtrlDeclaration();
	private ControllerDeclaration controller = new ControllerDeclaration();
	private TrafficLightCtrlDeclaration secondary = new TrafficLightCtrlDeclaration();
	// Port instances
	private Police police = new Police();
	private SecondaryOutput secondaryOutput = new SecondaryOutput();
	private PriorityOutput priorityOutput = new PriorityOutput();
	
	public CrossroadComponent() {
		init();
	}
	
	/** Enters the contained statemachines recursively. Should be used only be the container (composite system) class. */
	public void enter() {
		priority.enter();
		controller.enter();
		secondary.enter();
	}
	
	/** Creates the channel mappings and enters the wrapped statemachines. */
	private void init() {
		// Registration of simple channels
		controller.getPriorityPolice().registerListener(priority.getPoliceInterrupt());
		priority.getPoliceInterrupt().registerListener(controller.getPriorityPolice());
		controller.getSecondaryControl().registerListener(secondary.getControl());
		secondary.getControl().registerListener(controller.getSecondaryControl());
		controller.getSecondaryPolice().registerListener(secondary.getPoliceInterrupt());
		secondary.getPoliceInterrupt().registerListener(controller.getSecondaryPolice());
		controller.getPriorityControl().registerListener(priority.getControl());
		priority.getControl().registerListener(controller.getPriorityControl());
		// Registration of broadcast channels
		enter();
	}
	
	// Inner classes representing Ports
	public class Police implements PoliceInterruptInterface.Required {
	
		@Override
		public void raisePolice() {
			controller.getPoliceInterrupt().raisePolice();
		}
		
		@Override
		public void registerListener(PoliceInterruptInterface.Listener.Required listener) {
			controller.getPoliceInterrupt().registerListener(listener);
		}
	}
	
	@Override
	public Police getPolice() {
		return police;
	}
	
	public class SecondaryOutput implements LightCommandsInterface.Provided {
	
		
		@Override
		public boolean isRaisedDisplayRed() {
			return secondary.getLightCommands().isRaisedDisplayRed();
		}
		@Override
		public boolean isRaisedDisplayGreen() {
			return secondary.getLightCommands().isRaisedDisplayGreen();
		}
		@Override
		public boolean isRaisedDisplayNone() {
			return secondary.getLightCommands().isRaisedDisplayNone();
		}
		@Override
		public boolean isRaisedDisplayYellow() {
			return secondary.getLightCommands().isRaisedDisplayYellow();
		}
		@Override
		public void registerListener(LightCommandsInterface.Listener.Provided listener) {
			secondary.getLightCommands().registerListener(listener);
		}
	}
	
	@Override
	public SecondaryOutput getSecondaryOutput() {
		return secondaryOutput;
	}
	
	public class PriorityOutput implements LightCommandsInterface.Provided {
	
		
		@Override
		public boolean isRaisedDisplayRed() {
			return priority.getLightCommands().isRaisedDisplayRed();
		}
		@Override
		public boolean isRaisedDisplayGreen() {
			return priority.getLightCommands().isRaisedDisplayGreen();
		}
		@Override
		public boolean isRaisedDisplayNone() {
			return priority.getLightCommands().isRaisedDisplayNone();
		}
		@Override
		public boolean isRaisedDisplayYellow() {
			return priority.getLightCommands().isRaisedDisplayYellow();
		}
		@Override
		public void registerListener(LightCommandsInterface.Listener.Provided listener) {
			priority.getLightCommands().registerListener(listener);
		}
	}
	
	@Override
	public PriorityOutput getPriorityOutput() {
		return priorityOutput;
	}
	
	/** Changes the event queues of all component instances. Should be used only be the container (composite system) class. */
	public void changeEventQueue() {
		priority.changeEventQueue();
		controller.changeEventQueue();
		secondary.changeEventQueue();
	}
	
	/** Returns whether all message queues of the contained component instances are empty. 
	Should be used only be the container (composite system) class. */
	public boolean isEventQueueEmpty() {
		return priority.isEventQueueEmpty() && controller.isEventQueueEmpty() && secondary.isEventQueueEmpty();
	}
	
	/** Initiates cycle runs until all event queues of component instances are empty. */
	@Override
	public void runFullCycle() {
		do {
			runCycle();
		}
		while (!isEventQueueEmpty());
	}
	
	/** Changes event queues and initiates a cycle run. */
	@Override
	public void runCycle() {
		changeEventQueue();
		runCycleNoQueueChange();
	}
	
	/** Initiates a cycle run without changing the event queues. It is needed if this component is contained (wrapped) by another component.
	 Should be used only be the container (composite system) class. */
	public void runCycleNoQueueChange() {
		priority.runCycleNoQueueChange();
		controller.runCycleNoQueueChange();
		secondary.runCycleNoQueueChange();
	}
	
	public void setTimer(ITimer timer) {
		priority.setTimer(timer);
		controller.setTimer(timer);
		secondary.setTimer(timer);
	}
	
	/**  Getter for component instances, e.g. enabling to check their states. */
	public TrafficLightCtrlDeclaration getPriority() {
		return priority;
	}
	
	public ControllerDeclaration getController() {
		return controller;
	}
	
	public TrafficLightCtrlDeclaration getSecondary() {
		return secondary;
	}
	
	
}
