package hu.bme.mit.gamma.impl.monitor;

import java.util.LinkedList;
import java.util.Queue;

import hu.bme.mit.gamma.impl.event.*;
import hu.bme.mit.gamma.impl.interfaces.*;
import org.yakindu.scr.monitor.IMonitorStatemachine.SCIMonitorListener;
import org.yakindu.scr.monitor.MonitorStatemachine;
import org.yakindu.scr.monitor.MonitorStatemachine.State;

public class MonitorDeclaration implements MonitorDeclarationInterface {
	// The wrapped Yakindu statemachine
	private MonitorStatemachine monitorStatemachine = new MonitorStatemachine();
	// Port instances
	private LightInputs lightInputs = new LightInputs();
	private Monitor monitor = new Monitor();
	// Indicates which queue is active in this cycle
	private boolean chooseList = true;
	// Event queues for the synchronization of statecharts
	private Queue<Event> eventQueue1 = new LinkedList<Event>();
	private Queue<Event> eventQueue2 = new LinkedList<Event>();
	
	public MonitorDeclaration() {
		// Initializing and entering the wrapped statemachine
		monitorStatemachine.init();
		monitorStatemachine.enter();
	}
	
	/** Enters the statemachine. Should be used only be the container (composite system) class. */
	public void enter() {
		monitorStatemachine.enter();
	}
	
	/** Changes the event queues of the component instance. Should be used only be the container (composite system) class. */
	public void changeEventQueue() {
		chooseList = !chooseList;
	}
	
	/** Returns whether the eventQueue containing incoming messages is empty. Should be used only be the container (composite system) class. */
	public boolean isEventQueueEmpty() {
		return getInsertQueue().isEmpty();
	}
	
	/** Returns the event queue into which events should be put in the particular cycle. */
	private Queue<Event> getInsertQueue() {
		if (chooseList) {
			return eventQueue1;
		}
		return eventQueue2;
	}
	
	/** Returns the event queue from which events should be fetched in the particular cycle. */
	private Queue<Event> getProcessQueue() {
		return getInsertQueue() == eventQueue1 ? eventQueue2 : eventQueue1; 
	}
	
	/** Changes event queues and initiating a cycle run. */
	@Override
	public void runCycle() {
		changeEventQueue();
		runCycleNoQueueChange();
	}
	
	/** Initiates a cycle run without changing the event queues. It is needed if this component is contained (wrapped) by another component.
	Should be used only be the container (composite system) class. */
	public void runCycleNoQueueChange() {
		Queue<Event> eventQueue = getProcessQueue();
		while (!eventQueue.isEmpty()) {
				Event event = eventQueue.remove();
				switch (event.getEvent()) {
					case "LightInputs.DisplayRed": 
						monitorStatemachine.getSCILightInputs().raiseDisplayRed();
					break;
					case "LightInputs.DisplayGreen": 
						monitorStatemachine.getSCILightInputs().raiseDisplayGreen();
					break;
					case "LightInputs.DisplayNone": 
						monitorStatemachine.getSCILightInputs().raiseDisplayNone();
					break;
					case "LightInputs.DisplayYellow": 
						monitorStatemachine.getSCILightInputs().raiseDisplayYellow();
					break;
					default:
						throw new IllegalArgumentException("No such event!");
				}
		}
		monitorStatemachine.runCycle();
	}    		
	
	// Inner classes representing Ports
	public class LightInputs implements LightCommandsInterface.Required {

		@Override
		public void raiseDisplayRed() {
			getInsertQueue().add(new Event("LightInputs.DisplayRed", null));
		}
		@Override
		public void raiseDisplayGreen() {
			getInsertQueue().add(new Event("LightInputs.DisplayGreen", null));
		}
		@Override
		public void raiseDisplayNone() {
			getInsertQueue().add(new Event("LightInputs.DisplayNone", null));
		}
		@Override
		public void raiseDisplayYellow() {
			getInsertQueue().add(new Event("LightInputs.DisplayYellow", null));
		}

		@Override
		public void registerListener(LightCommandsInterface.Listener.Required listener) {
		}

	}
	
	@Override
	public LightInputs getLightInputs() {
		return lightInputs;
	}
	
	public class Monitor implements MonitorInterface.Provided {


		@Override
		public boolean isRaisedError() {
			return monitorStatemachine.getSCIMonitor().isRaisedError();
		}
		@Override
		public void registerListener(MonitorInterface.Listener.Provided listener) {
			//
			monitorStatemachine.getSCIMonitor().getListeners().add(new SCIMonitorListener() {
				@Override
				public void onErrorRaised() {
					listener.raiseError();
				}
			});
		}

	}
	
	@Override
	public Monitor getMonitor() {
		return monitor;
	}
	
	/** Checks whether the wrapped statemachine is in the given state. */
	public boolean isStateActive(State state) {
		return monitorStatemachine.isStateActive(state);
	}
	
	
}
