package hu.bme.mit.gamma.impl.monitoredcrossroad;

import org.yakindu.scr.ITimer;

import hu.bme.mit.gamma.impl.interfaces.*;    	
import hu.bme.mit.gamma.impl.crossroad.*;
import hu.bme.mit.gamma.impl.monitor.*;

public class MonitoredCrossroadComponent implements MonitoredCrossroadComponentInterface {			
	// ComponentDeclaration instances
	private MonitorDeclaration monitor = new MonitorDeclaration();
	private CrossroadComponent crossroad = new CrossroadComponent();
	// Port instances
	private MonitorOutput monitorOutput = new MonitorOutput();
	private Police police = new Police();
	private PriorityOutput priorityOutput = new PriorityOutput();
	private SecondaryOutput secondaryOutput = new SecondaryOutput();
	
	public MonitoredCrossroadComponent() {
		init();
	}
	
	/** Enters the contained statemachines recursively. Should be used only be the container (composite system) class. */
	public void enter() {
		monitor.enter();
		crossroad.enter();
	}
	
	/** Creates the channel mappings and enters the wrapped statemachines. */
	private void init() {
		// Registration of simple channels
		crossroad.getPriorityOutput().registerListener(monitor.getLightInputs());
		monitor.getLightInputs().registerListener(crossroad.getPriorityOutput());
		// Registration of broadcast channels
		enter();
	}
	
	// Inner classes representing Ports
	public class MonitorOutput implements MonitorInterface.Provided {
	
		
		@Override
		public boolean isRaisedError() {
			return monitor.getMonitor().isRaisedError();
		}
		@Override
		public void registerListener(MonitorInterface.Listener.Provided listener) {
			monitor.getMonitor().registerListener(listener);
		}
	}
	
	@Override
	public MonitorOutput getMonitorOutput() {
		return monitorOutput;
	}
	
	public class Police implements PoliceInterruptInterface.Required {
	
		@Override
		public void raisePolice() {
			crossroad.getPolice().raisePolice();
		}
		
		@Override
		public void registerListener(PoliceInterruptInterface.Listener.Required listener) {
			crossroad.getPolice().registerListener(listener);
		}
	}
	
	@Override
	public Police getPolice() {
		return police;
	}
	
	public class PriorityOutput implements LightCommandsInterface.Provided {
	
		
		@Override
		public boolean isRaisedDisplayRed() {
			return crossroad.getPriorityOutput().isRaisedDisplayRed();
		}
		@Override
		public boolean isRaisedDisplayGreen() {
			return crossroad.getPriorityOutput().isRaisedDisplayGreen();
		}
		@Override
		public boolean isRaisedDisplayNone() {
			return crossroad.getPriorityOutput().isRaisedDisplayNone();
		}
		@Override
		public boolean isRaisedDisplayYellow() {
			return crossroad.getPriorityOutput().isRaisedDisplayYellow();
		}
		@Override
		public void registerListener(LightCommandsInterface.Listener.Provided listener) {
			crossroad.getPriorityOutput().registerListener(listener);
		}
	}
	
	@Override
	public PriorityOutput getPriorityOutput() {
		return priorityOutput;
	}
	
	public class SecondaryOutput implements LightCommandsInterface.Provided {
	
		
		@Override
		public boolean isRaisedDisplayRed() {
			return crossroad.getSecondaryOutput().isRaisedDisplayRed();
		}
		@Override
		public boolean isRaisedDisplayGreen() {
			return crossroad.getSecondaryOutput().isRaisedDisplayGreen();
		}
		@Override
		public boolean isRaisedDisplayNone() {
			return crossroad.getSecondaryOutput().isRaisedDisplayNone();
		}
		@Override
		public boolean isRaisedDisplayYellow() {
			return crossroad.getSecondaryOutput().isRaisedDisplayYellow();
		}
		@Override
		public void registerListener(LightCommandsInterface.Listener.Provided listener) {
			crossroad.getSecondaryOutput().registerListener(listener);
		}
	}
	
	@Override
	public SecondaryOutput getSecondaryOutput() {
		return secondaryOutput;
	}
	
	/** Changes the event queues of all component instances. Should be used only be the container (composite system) class. */
	public void changeEventQueue() {
		monitor.changeEventQueue();
		crossroad.changeEventQueue();
	}
	
	/** Returns whether all message queues of the contained component instances are empty. 
	Should be used only be the container (composite system) class. */
	public boolean isEventQueueEmpty() {
		return monitor.isEventQueueEmpty() && crossroad.isEventQueueEmpty();
	}
	
	/** Initiates cycle runs until all event queues of component instances are empty. */
	@Override
	public void runFullCycle() {
		do {
			runCycle();
		}
		while (!isEventQueueEmpty());
	}
	
	/** Changes event queues and initiates a cycle run. */
	@Override
	public void runCycle() {
		changeEventQueue();
		runCycleNoQueueChange();
	}
	
	/** Initiates a cycle run without changing the event queues. It is needed if this component is contained (wrapped) by another component.
	 Should be used only be the container (composite system) class. */
	public void runCycleNoQueueChange() {
		monitor.runCycleNoQueueChange();
		crossroad.runCycleNoQueueChange();
	}
	
	public void setTimer(ITimer timer) {
		crossroad.setTimer(timer);
	}
	
	/**  Getter for component instances, e.g. enabling to check their states. */
	public MonitorDeclaration getMonitor() {
		return monitor;
	}
	
	public CrossroadComponent getCrossroad() {
		return crossroad;
	}
	
	
}
