package hu.bme.mit.gamma.impl.trafficlightctrl;

import java.util.LinkedList;
import java.util.Queue;

import hu.bme.mit.gamma.impl.event.*;
import hu.bme.mit.gamma.impl.interfaces.*;
import org.yakindu.scr.trafficlightctrl.ITrafficLightCtrlStatemachine.SCILightCommandsListener;
import org.yakindu.scr.TimerService;
import org.yakindu.scr.ITimer;
import org.yakindu.scr.trafficlightctrl.TrafficLightCtrlStatemachine;
import org.yakindu.scr.trafficlightctrl.TrafficLightCtrlStatemachine.State;

public class TrafficLightCtrlDeclaration implements TrafficLightCtrlDeclarationInterface {
	// The wrapped Yakindu statemachine
	private TrafficLightCtrlStatemachine trafficLightCtrlStatemachine = new TrafficLightCtrlStatemachine();
	// Port instances
	private LightCommands lightCommands = new LightCommands();
	private Control control = new Control();
	private PoliceInterrupt policeInterrupt = new PoliceInterrupt();
	// Indicates which queue is active in this cycle
	private boolean chooseList = true;
	// Event queues for the synchronization of statecharts
	private Queue<Event> eventQueue1 = new LinkedList<Event>();
	private Queue<Event> eventQueue2 = new LinkedList<Event>();
	
	public TrafficLightCtrlDeclaration() {
		// Initializing and entering the wrapped statemachine
		trafficLightCtrlStatemachine.setTimer(new TimerService());
		trafficLightCtrlStatemachine.init();
		trafficLightCtrlStatemachine.enter();
	}
	
	/** Enters the statemachine. Should be used only be the container (composite system) class. */
	public void enter() {
		trafficLightCtrlStatemachine.enter();
	}
	
	/** Changes the event queues of the component instance. Should be used only be the container (composite system) class. */
	public void changeEventQueue() {
		chooseList = !chooseList;
	}
	
	/** Returns whether the eventQueue containing incoming messages is empty. Should be used only be the container (composite system) class. */
	public boolean isEventQueueEmpty() {
		return getInsertQueue().isEmpty();
	}
	
	/** Returns the event queue into which events should be put in the particular cycle. */
	private Queue<Event> getInsertQueue() {
		if (chooseList) {
			return eventQueue1;
		}
		return eventQueue2;
	}
	
	/** Returns the event queue from which events should be fetched in the particular cycle. */
	private Queue<Event> getProcessQueue() {
		return getInsertQueue() == eventQueue1 ? eventQueue2 : eventQueue1; 
	}
	
	/** Changes event queues and initiating a cycle run. */
	@Override
	public void runCycle() {
		changeEventQueue();
		runCycleNoQueueChange();
	}
	
	/** Initiates a cycle run without changing the event queues. It is needed if this component is contained (wrapped) by another component.
	Should be used only be the container (composite system) class. */
	public void runCycleNoQueueChange() {
		Queue<Event> eventQueue = getProcessQueue();
		while (!eventQueue.isEmpty()) {
				Event event = eventQueue.remove();
				switch (event.getEvent()) {
					case "Control.Toggle": 
						trafficLightCtrlStatemachine.getSCIControl().raiseToggle();
					break;
					case "PoliceInterrupt.Police": 
						trafficLightCtrlStatemachine.getSCIPoliceInterrupt().raisePolice();
					break;
					default:
						throw new IllegalArgumentException("No such event!");
				}
		}
		trafficLightCtrlStatemachine.runCycle();
	}    		
	
	// Inner classes representing Ports
	public class LightCommands implements LightCommandsInterface.Provided {


		@Override
		public boolean isRaisedDisplayRed() {
			return trafficLightCtrlStatemachine.getSCILightCommands().isRaisedDisplayRed();
		}
		@Override
		public boolean isRaisedDisplayGreen() {
			return trafficLightCtrlStatemachine.getSCILightCommands().isRaisedDisplayGreen();
		}
		@Override
		public boolean isRaisedDisplayNone() {
			return trafficLightCtrlStatemachine.getSCILightCommands().isRaisedDisplayNone();
		}
		@Override
		public boolean isRaisedDisplayYellow() {
			return trafficLightCtrlStatemachine.getSCILightCommands().isRaisedDisplayYellow();
		}
		@Override
		public void registerListener(LightCommandsInterface.Listener.Provided listener) {
			//
			trafficLightCtrlStatemachine.getSCILightCommands().getListeners().add(new SCILightCommandsListener() {
				@Override
				public void onDisplayRedRaised() {
					listener.raiseDisplayRed();
				}
				
				@Override
				public void onDisplayGreenRaised() {
					listener.raiseDisplayGreen();
				}
				
				@Override
				public void onDisplayNoneRaised() {
					listener.raiseDisplayNone();
				}
				
				@Override
				public void onDisplayYellowRaised() {
					listener.raiseDisplayYellow();
				}
			});
		}

	}
	
	@Override
	public LightCommands getLightCommands() {
		return lightCommands;
	}
	
	public class Control implements ControlInterface.Required {

		@Override
		public void raiseToggle() {
			getInsertQueue().add(new Event("Control.Toggle", null));
		}

		@Override
		public void registerListener(ControlInterface.Listener.Required listener) {
		}

	}
	
	@Override
	public Control getControl() {
		return control;
	}
	
	public class PoliceInterrupt implements PoliceInterruptInterface.Required {

		@Override
		public void raisePolice() {
			getInsertQueue().add(new Event("PoliceInterrupt.Police", null));
		}

		@Override
		public void registerListener(PoliceInterruptInterface.Listener.Required listener) {
		}

	}
	
	@Override
	public PoliceInterrupt getPoliceInterrupt() {
		return policeInterrupt;
	}
	
	/** Checks whether the wrapped statemachine is in the given state. */
	public boolean isStateActive(State state) {
		return trafficLightCtrlStatemachine.isStateActive(state);
	}
	
	public void setTimer(ITimer timer) {
		trafficLightCtrlStatemachine.setTimer(timer);
	}
	
}
