package hu.bme.mit.gamma.impl.monitoredcrossroad;

import hu.bme.mit.gamma.impl.timer.VirtualTimerService;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.assertEquals;

import org.junit.Before;
import org.junit.Test;

public class ExecutionTraceSimulation0 {
	
	private static MonitoredCrossroadComponent monitoredCrossroadComponent;
	private static VirtualTimerService timer;
	
	@Before
	public void init() {
		monitoredCrossroadComponent = new MonitoredCrossroadComponent();
		timer = new VirtualTimerService();
		monitoredCrossroadComponent.setTimer(timer);
		// Needed to set the virtual timer
		monitoredCrossroadComponent.enter();
	}
	
	@Test
	public void step1() {
		assertTrue(monitoredCrossroadComponent.getPriorityOutput().isRaisedDisplayRed());
		assertTrue(monitoredCrossroadComponent.getSecondaryOutput().isRaisedDisplayRed());
		assertTrue(monitoredCrossroadComponent.getCrossroad().getPriority().isStateActive(org.yakindu.scr.trafficlightctrl.TrafficLightCtrlStatemachine.State.main_region_Normal_normal_Red));
		assertTrue(monitoredCrossroadComponent.getCrossroad().getController().isStateActive(org.yakindu.scr.controller.ControllerStatemachine.State.main_region_Operating));
		assertTrue(monitoredCrossroadComponent.getCrossroad().getSecondary().isStateActive(org.yakindu.scr.trafficlightctrl.TrafficLightCtrlStatemachine.State.main_region_Normal));
		assertTrue(monitoredCrossroadComponent.getCrossroad().getSecondary().isStateActive(org.yakindu.scr.trafficlightctrl.TrafficLightCtrlStatemachine.State.main_region_Normal_normal_Red));
		assertTrue(monitoredCrossroadComponent.getCrossroad().getPriority().isStateActive(org.yakindu.scr.trafficlightctrl.TrafficLightCtrlStatemachine.State.main_region_Normal));
		assertTrue(monitoredCrossroadComponent.getCrossroad().getController().isStateActive(org.yakindu.scr.controller.ControllerStatemachine.State.main_region_Operating_operating_Init));
		assertTrue(monitoredCrossroadComponent.getMonitor().isStateActive(org.yakindu.scr.monitor.MonitorStatemachine.State.main_region_Other));
	}
	
	@Test
	public void step2() {
		step1();
		timer.elapse(2000);
		monitoredCrossroadComponent.runCycle();
		assertTrue(monitoredCrossroadComponent.getPriorityOutput().isRaisedDisplayGreen());
		assertTrue(monitoredCrossroadComponent.getMonitor().isStateActive(org.yakindu.scr.monitor.MonitorStatemachine.State.main_region_Red));
		assertTrue(monitoredCrossroadComponent.getCrossroad().getController().isStateActive(org.yakindu.scr.controller.ControllerStatemachine.State.main_region_Operating));
		assertTrue(monitoredCrossroadComponent.getCrossroad().getSecondary().isStateActive(org.yakindu.scr.trafficlightctrl.TrafficLightCtrlStatemachine.State.main_region_Normal));
		assertTrue(monitoredCrossroadComponent.getCrossroad().getController().isStateActive(org.yakindu.scr.controller.ControllerStatemachine.State.main_region_Operating_operating_PriorityPrepares));
		assertTrue(monitoredCrossroadComponent.getCrossroad().getSecondary().isStateActive(org.yakindu.scr.trafficlightctrl.TrafficLightCtrlStatemachine.State.main_region_Normal_normal_Red));
		assertTrue(monitoredCrossroadComponent.getCrossroad().getPriority().isStateActive(org.yakindu.scr.trafficlightctrl.TrafficLightCtrlStatemachine.State.main_region_Normal));
		assertTrue(monitoredCrossroadComponent.getCrossroad().getPriority().isStateActive(org.yakindu.scr.trafficlightctrl.TrafficLightCtrlStatemachine.State.main_region_Normal_normal_Green));
	}
	
	@Test
	public void step3() {
		step2();
		timer.elapse(1000);
		monitoredCrossroadComponent.runCycle();
		assertTrue(monitoredCrossroadComponent.getPriorityOutput().isRaisedDisplayYellow());
		assertTrue(monitoredCrossroadComponent.getCrossroad().getController().isStateActive(org.yakindu.scr.controller.ControllerStatemachine.State.main_region_Operating));
		assertTrue(monitoredCrossroadComponent.getCrossroad().getController().isStateActive(org.yakindu.scr.controller.ControllerStatemachine.State.main_region_Operating_operating_Secondary));
		assertTrue(monitoredCrossroadComponent.getCrossroad().getSecondary().isStateActive(org.yakindu.scr.trafficlightctrl.TrafficLightCtrlStatemachine.State.main_region_Normal));
		assertTrue(monitoredCrossroadComponent.getCrossroad().getSecondary().isStateActive(org.yakindu.scr.trafficlightctrl.TrafficLightCtrlStatemachine.State.main_region_Normal_normal_Red));
		assertTrue(monitoredCrossroadComponent.getCrossroad().getPriority().isStateActive(org.yakindu.scr.trafficlightctrl.TrafficLightCtrlStatemachine.State.main_region_Normal_normal_Yellow));
		assertTrue(monitoredCrossroadComponent.getCrossroad().getPriority().isStateActive(org.yakindu.scr.trafficlightctrl.TrafficLightCtrlStatemachine.State.main_region_Normal));
		assertTrue(monitoredCrossroadComponent.getMonitor().isStateActive(org.yakindu.scr.monitor.MonitorStatemachine.State.main_region_Green));
	}
	
	@Test
	public void step4() {
		step3();
		timer.elapse(2000);
		monitoredCrossroadComponent.runCycle();
		assertTrue(monitoredCrossroadComponent.getPriorityOutput().isRaisedDisplayRed());
		assertTrue(monitoredCrossroadComponent.getSecondaryOutput().isRaisedDisplayGreen());
		assertTrue(monitoredCrossroadComponent.getCrossroad().getPriority().isStateActive(org.yakindu.scr.trafficlightctrl.TrafficLightCtrlStatemachine.State.main_region_Normal_normal_Red));
		assertTrue(monitoredCrossroadComponent.getCrossroad().getController().isStateActive(org.yakindu.scr.controller.ControllerStatemachine.State.main_region_Operating));
		assertTrue(monitoredCrossroadComponent.getCrossroad().getSecondary().isStateActive(org.yakindu.scr.trafficlightctrl.TrafficLightCtrlStatemachine.State.main_region_Normal));
		assertTrue(monitoredCrossroadComponent.getCrossroad().getPriority().isStateActive(org.yakindu.scr.trafficlightctrl.TrafficLightCtrlStatemachine.State.main_region_Normal));
		assertTrue(monitoredCrossroadComponent.getCrossroad().getSecondary().isStateActive(org.yakindu.scr.trafficlightctrl.TrafficLightCtrlStatemachine.State.main_region_Normal_normal_Green));
		assertTrue(monitoredCrossroadComponent.getCrossroad().getController().isStateActive(org.yakindu.scr.controller.ControllerStatemachine.State.main_region_Operating_operating_SecondaryPrepares));
		assertTrue(monitoredCrossroadComponent.getMonitor().isStateActive(org.yakindu.scr.monitor.MonitorStatemachine.State.main_region_Other));
	}
	
}
