package hu.bme.mit.gamma.impl.controller;

import java.util.LinkedList;
import java.util.Queue;

import hu.bme.mit.gamma.impl.event.*;
import hu.bme.mit.gamma.impl.interfaces.*;
import org.yakindu.scr.controller.IControllerStatemachine.SCISecondaryPoliceListener;
import org.yakindu.scr.controller.IControllerStatemachine.SCIPriorityControlListener;
import org.yakindu.scr.controller.IControllerStatemachine.SCIPriorityPoliceListener;
import org.yakindu.scr.controller.IControllerStatemachine.SCISecondaryControlListener;
import org.yakindu.scr.TimerService;
import org.yakindu.scr.ITimer;
import org.yakindu.scr.controller.ControllerStatemachine;
import org.yakindu.scr.controller.ControllerStatemachine.State;

public class ControllerDeclaration implements ControllerDeclarationInterface {
	// The wrapped Yakindu statemachine
	private ControllerStatemachine controllerStatemachine = new ControllerStatemachine();
	// Port instances
	private SecondaryPolice secondaryPolice = new SecondaryPolice();
	private PriorityControl priorityControl = new PriorityControl();
	private PoliceInterrupt policeInterrupt = new PoliceInterrupt();
	private PriorityPolice priorityPolice = new PriorityPolice();
	private SecondaryControl secondaryControl = new SecondaryControl();
	// Indicates which queue is active in this cycle
	private boolean chooseList = true;
	// Event queues for the synchronization of statecharts
	private Queue<Event> eventQueue1 = new LinkedList<Event>();
	private Queue<Event> eventQueue2 = new LinkedList<Event>();
	
	public ControllerDeclaration() {
		// Initializing and entering the wrapped statemachine
		controllerStatemachine.setTimer(new TimerService());
		controllerStatemachine.init();
		controllerStatemachine.enter();
	}
	
	/** Enters the statemachine. Should be used only be the container (composite system) class. */
	public void enter() {
		controllerStatemachine.enter();
	}
	
	/** Changes the event queues of the component instance. Should be used only be the container (composite system) class. */
	public void changeEventQueue() {
		chooseList = !chooseList;
	}
	
	/** Returns whether the eventQueue containing incoming messages is empty. Should be used only be the container (composite system) class. */
	public boolean isEventQueueEmpty() {
		return getInsertQueue().isEmpty();
	}
	
	/** Returns the event queue into which events should be put in the particular cycle. */
	private Queue<Event> getInsertQueue() {
		if (chooseList) {
			return eventQueue1;
		}
		return eventQueue2;
	}
	
	/** Returns the event queue from which events should be fetched in the particular cycle. */
	private Queue<Event> getProcessQueue() {
		return getInsertQueue() == eventQueue1 ? eventQueue2 : eventQueue1; 
	}
	
	/** Changes event queues and initiating a cycle run. */
	@Override
	public void runCycle() {
		changeEventQueue();
		runCycleNoQueueChange();
	}
	
	/** Initiates a cycle run without changing the event queues. It is needed if this component is contained (wrapped) by another component.
	Should be used only be the container (composite system) class. */
	public void runCycleNoQueueChange() {
		Queue<Event> eventQueue = getProcessQueue();
		while (!eventQueue.isEmpty()) {
				Event event = eventQueue.remove();
				switch (event.getEvent()) {
					case "PoliceInterrupt.Police": 
						controllerStatemachine.getSCIPoliceInterrupt().raisePolice();
					break;
					default:
						throw new IllegalArgumentException("No such event!");
				}
		}
		controllerStatemachine.runCycle();
	}    		
	
	// Inner classes representing Ports
	public class SecondaryPolice implements PoliceInterruptInterface.Provided {


		@Override
		public boolean isRaisedPolice() {
			return controllerStatemachine.getSCISecondaryPolice().isRaisedPolice();
		}
		@Override
		public void registerListener(PoliceInterruptInterface.Listener.Provided listener) {
			//
			controllerStatemachine.getSCISecondaryPolice().getListeners().add(new SCISecondaryPoliceListener() {
				@Override
				public void onPoliceRaised() {
					listener.raisePolice();
				}
			});
		}

	}
	
	@Override
	public SecondaryPolice getSecondaryPolice() {
		return secondaryPolice;
	}
	
	public class PriorityControl implements ControlInterface.Provided {


		@Override
		public boolean isRaisedToggle() {
			return controllerStatemachine.getSCIPriorityControl().isRaisedToggle();
		}
		@Override
		public void registerListener(ControlInterface.Listener.Provided listener) {
			//
			controllerStatemachine.getSCIPriorityControl().getListeners().add(new SCIPriorityControlListener() {
				@Override
				public void onToggleRaised() {
					listener.raiseToggle();
				}
			});
		}

	}
	
	@Override
	public PriorityControl getPriorityControl() {
		return priorityControl;
	}
	
	public class PoliceInterrupt implements PoliceInterruptInterface.Required {

		@Override
		public void raisePolice() {
			getInsertQueue().add(new Event("PoliceInterrupt.Police", null));
		}

		@Override
		public void registerListener(PoliceInterruptInterface.Listener.Required listener) {
		}

	}
	
	@Override
	public PoliceInterrupt getPoliceInterrupt() {
		return policeInterrupt;
	}
	
	public class PriorityPolice implements PoliceInterruptInterface.Provided {


		@Override
		public boolean isRaisedPolice() {
			return controllerStatemachine.getSCIPriorityPolice().isRaisedPolice();
		}
		@Override
		public void registerListener(PoliceInterruptInterface.Listener.Provided listener) {
			//
			controllerStatemachine.getSCIPriorityPolice().getListeners().add(new SCIPriorityPoliceListener() {
				@Override
				public void onPoliceRaised() {
					listener.raisePolice();
				}
			});
		}

	}
	
	@Override
	public PriorityPolice getPriorityPolice() {
		return priorityPolice;
	}
	
	public class SecondaryControl implements ControlInterface.Provided {


		@Override
		public boolean isRaisedToggle() {
			return controllerStatemachine.getSCISecondaryControl().isRaisedToggle();
		}
		@Override
		public void registerListener(ControlInterface.Listener.Provided listener) {
			//
			controllerStatemachine.getSCISecondaryControl().getListeners().add(new SCISecondaryControlListener() {
				@Override
				public void onToggleRaised() {
					listener.raiseToggle();
				}
			});
		}

	}
	
	@Override
	public SecondaryControl getSecondaryControl() {
		return secondaryControl;
	}
	
	/** Checks whether the wrapped statemachine is in the given state. */
	public boolean isStateActive(State state) {
		return controllerStatemachine.isStateActive(state);
	}
	
	public void setTimer(ITimer timer) {
		controllerStatemachine.setTimer(timer);
	}
	
}
