package hu.bme.mit.gamma.impl.crossroad;

import hu.bme.mit.gamma.impl.timer.VirtualTimerService;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.assertEquals;

import org.junit.Before;
import org.junit.Test;

public class ExecutionTraceSimulation0 {
	
	private static CrossroadComponent crossroadComponent;
	private static VirtualTimerService timer;
	
	@Before
	public void init() {
		crossroadComponent = new CrossroadComponent();
		timer = new VirtualTimerService();
		crossroadComponent.setTimer(timer);
		// Needed to set the virtual timer
		crossroadComponent.enter();
	}
	
	@Test
	public void step1() {
		assertTrue(crossroadComponent.getPriorityOutput().isRaisedDisplayRed());
		assertTrue(crossroadComponent.getSecondaryOutput().isRaisedDisplayRed());
		assertTrue(crossroadComponent.getPriority().isStateActive(org.yakindu.scr.trafficlightctrl.TrafficLightCtrlStatemachine.State.main_region_Normal_normal_Red));
		assertTrue(crossroadComponent.getController().isStateActive(org.yakindu.scr.controller.ControllerStatemachine.State.main_region_Operating));
		assertTrue(crossroadComponent.getSecondary().isStateActive(org.yakindu.scr.trafficlightctrl.TrafficLightCtrlStatemachine.State.main_region_Normal));
		assertTrue(crossroadComponent.getController().isStateActive(org.yakindu.scr.controller.ControllerStatemachine.State.main_region_Operating_operating_Init));
		assertTrue(crossroadComponent.getSecondary().isStateActive(org.yakindu.scr.trafficlightctrl.TrafficLightCtrlStatemachine.State.main_region_Normal_normal_Red));
		assertTrue(crossroadComponent.getPriority().isStateActive(org.yakindu.scr.trafficlightctrl.TrafficLightCtrlStatemachine.State.main_region_Normal));
	}
	
	@Test
	public void step2() {
		step1();
		crossroadComponent.getPolice().raisePolice();
		crossroadComponent.runCycle();
		assertTrue(crossroadComponent.getPriorityOutput().isRaisedDisplayGreen());
		assertTrue(crossroadComponent.getController().isStateActive(org.yakindu.scr.controller.ControllerStatemachine.State.main_region_Operating));
		assertTrue(crossroadComponent.getSecondary().isStateActive(org.yakindu.scr.trafficlightctrl.TrafficLightCtrlStatemachine.State.main_region_Normal));
		assertTrue(crossroadComponent.getController().isStateActive(org.yakindu.scr.controller.ControllerStatemachine.State.main_region_Operating_operating_Init));
		assertTrue(crossroadComponent.getSecondary().isStateActive(org.yakindu.scr.trafficlightctrl.TrafficLightCtrlStatemachine.State.main_region_Normal_normal_Red));
		assertTrue(crossroadComponent.getPriority().isStateActive(org.yakindu.scr.trafficlightctrl.TrafficLightCtrlStatemachine.State.main_region_Normal_normal_Green));
		assertTrue(crossroadComponent.getPriority().isStateActive(org.yakindu.scr.trafficlightctrl.TrafficLightCtrlStatemachine.State.main_region_Normal));
	}
	
	@Test
	public void step3() {
		step2();
		timer.elapse(2000);
		crossroadComponent.runCycle();
		assertTrue(crossroadComponent.getPriorityOutput().isRaisedDisplayYellow());
		assertTrue(crossroadComponent.getSecondaryOutput().isRaisedDisplayYellow());
		assertTrue(crossroadComponent.getController().isStateActive(org.yakindu.scr.controller.ControllerStatemachine.State.main_region_Operating));
		assertTrue(crossroadComponent.getPriority().isStateActive(org.yakindu.scr.trafficlightctrl.TrafficLightCtrlStatemachine.State.main_region_Interrupted_interrupted_Yellow));
		assertTrue(crossroadComponent.getPriority().isStateActive(org.yakindu.scr.trafficlightctrl.TrafficLightCtrlStatemachine.State.main_region_Interrupted));
		assertTrue(crossroadComponent.getController().isStateActive(org.yakindu.scr.controller.ControllerStatemachine.State.main_region_Operating_operating_PriorityPrepares));
		assertTrue(crossroadComponent.getSecondary().isStateActive(org.yakindu.scr.trafficlightctrl.TrafficLightCtrlStatemachine.State.main_region_Interrupted));
		assertTrue(crossroadComponent.getSecondary().isStateActive(org.yakindu.scr.trafficlightctrl.TrafficLightCtrlStatemachine.State.main_region_Interrupted_interrupted_Yellow));
	}
	
	@Test
	public void step4() {
		step3();
		timer.elapse(1000);
		crossroadComponent.runCycle();
		assertTrue(crossroadComponent.getPriorityOutput().isRaisedDisplayNone());
		assertTrue(crossroadComponent.getSecondaryOutput().isRaisedDisplayNone());
		assertTrue(crossroadComponent.getController().isStateActive(org.yakindu.scr.controller.ControllerStatemachine.State.main_region_Operating));
		assertTrue(crossroadComponent.getPriority().isStateActive(org.yakindu.scr.trafficlightctrl.TrafficLightCtrlStatemachine.State.main_region_Interrupted));
		assertTrue(crossroadComponent.getSecondary().isStateActive(org.yakindu.scr.trafficlightctrl.TrafficLightCtrlStatemachine.State.main_region_Interrupted_interrupted_Black));
		assertTrue(crossroadComponent.getPriority().isStateActive(org.yakindu.scr.trafficlightctrl.TrafficLightCtrlStatemachine.State.main_region_Interrupted_interrupted_Black));
		assertTrue(crossroadComponent.getController().isStateActive(org.yakindu.scr.controller.ControllerStatemachine.State.main_region_Operating_operating_Secondary));
		assertTrue(crossroadComponent.getSecondary().isStateActive(org.yakindu.scr.trafficlightctrl.TrafficLightCtrlStatemachine.State.main_region_Interrupted));
	}
	
	@Test
	public void step5() {
		step4();
		crossroadComponent.getPolice().raisePolice();
		crossroadComponent.runCycle();
		assertTrue(crossroadComponent.getController().isStateActive(org.yakindu.scr.controller.ControllerStatemachine.State.main_region_Operating));
		assertTrue(crossroadComponent.getPriority().isStateActive(org.yakindu.scr.trafficlightctrl.TrafficLightCtrlStatemachine.State.main_region_Interrupted));
		assertTrue(crossroadComponent.getSecondary().isStateActive(org.yakindu.scr.trafficlightctrl.TrafficLightCtrlStatemachine.State.main_region_Interrupted_interrupted_Black));
		assertTrue(crossroadComponent.getPriority().isStateActive(org.yakindu.scr.trafficlightctrl.TrafficLightCtrlStatemachine.State.main_region_Interrupted_interrupted_Black));
		assertTrue(crossroadComponent.getController().isStateActive(org.yakindu.scr.controller.ControllerStatemachine.State.main_region_Operating_operating_Secondary));
		assertTrue(crossroadComponent.getSecondary().isStateActive(org.yakindu.scr.trafficlightctrl.TrafficLightCtrlStatemachine.State.main_region_Interrupted));
	}
	
	@Test
	public void step6() {
		step5();
		timer.elapse(2000);
		crossroadComponent.runCycle();
		assertTrue(crossroadComponent.getSecondaryOutput().isRaisedDisplayRed());
		assertTrue(crossroadComponent.getPriorityOutput().isRaisedDisplayGreen());
		assertTrue(crossroadComponent.getController().isStateActive(org.yakindu.scr.controller.ControllerStatemachine.State.main_region_Operating));
		assertTrue(crossroadComponent.getSecondary().isStateActive(org.yakindu.scr.trafficlightctrl.TrafficLightCtrlStatemachine.State.main_region_Normal));
		assertTrue(crossroadComponent.getController().isStateActive(org.yakindu.scr.controller.ControllerStatemachine.State.main_region_Operating_operating_SecondaryPrepares));
		assertTrue(crossroadComponent.getSecondary().isStateActive(org.yakindu.scr.trafficlightctrl.TrafficLightCtrlStatemachine.State.main_region_Normal_normal_Red));
		assertTrue(crossroadComponent.getPriority().isStateActive(org.yakindu.scr.trafficlightctrl.TrafficLightCtrlStatemachine.State.main_region_Normal_normal_Green));
		assertTrue(crossroadComponent.getPriority().isStateActive(org.yakindu.scr.trafficlightctrl.TrafficLightCtrlStatemachine.State.main_region_Normal));
	}
	
	@Test
	public void step7() {
		step6();
		crossroadComponent.runCycle();
		assertTrue(crossroadComponent.getSecondaryOutput().isRaisedDisplayGreen());
		assertTrue(crossroadComponent.getSecondary().isStateActive(org.yakindu.scr.trafficlightctrl.TrafficLightCtrlStatemachine.State.main_region_Normal_normal_Green));
		assertTrue(crossroadComponent.getController().isStateActive(org.yakindu.scr.controller.ControllerStatemachine.State.main_region_Operating));
		assertTrue(crossroadComponent.getSecondary().isStateActive(org.yakindu.scr.trafficlightctrl.TrafficLightCtrlStatemachine.State.main_region_Normal));
		assertTrue(crossroadComponent.getController().isStateActive(org.yakindu.scr.controller.ControllerStatemachine.State.main_region_Operating_operating_SecondaryPrepares));
		assertTrue(crossroadComponent.getPriority().isStateActive(org.yakindu.scr.trafficlightctrl.TrafficLightCtrlStatemachine.State.main_region_Normal_normal_Green));
		assertTrue(crossroadComponent.getPriority().isStateActive(org.yakindu.scr.trafficlightctrl.TrafficLightCtrlStatemachine.State.main_region_Normal));
	}
	
}
